@@ -1,10 +1,13 @@
 #![feature(unboxed_closures, fn_traits, thread_id_value, const_fn_fn_ptr_basics)]
 
+pub use entry::*;
 pub use handler::*;
-pub use sandbox::*;
+pub use instance::*;
+pub use wasmy_abi::*;
 
 mod handler;
-mod sandbox;
+mod instance;
+mod entry;
 
 #[cfg(test)]
 mod tests {
